function Select_tractography_rnh
% Rafael Neto Henriques July 2013
% Last update: 06/04/2015

archstr = computer('arch');
st_comp=archstr(1:3);
if strcmp(st_comp,'win')
    sc='\';
else
    sc='/';
end

hfig=figure;
set(hfig,'color',[0.8 0.8 0.8],'name','Tractography Selection',...
    'Units','normalized','position',[0.5,0.2,0.3,0.3],'Toolbar',...
    'none','NumberTitle','off','MenuBar','none');

upind=uibuttongroup('FontSize',12,...
    'BackgroundColor',[0.93 0.93 0.93],'Units','Normalized',...
    'Position',[.025 .2042 .95 0.7708],'HighlightColor','b',...
    'ForegroundColor','b');

% Button after Method's Selected
uicontrol('Units','Normalized','Position',[0.025,0.025,0.95,0.925/6],...
    'String','Process','Callback',{@Call_Fit});

% Select Fit Method
uicontrol('Style','Radio', 'Units','normalized','Parent',upind, ...
    'HandleVisibility','off', 'Position',[0.05 13/17 0.9 3/17], ...
    'String','DTI tractography', 'Tag','tDTI','FontWeight','bold')
uicontrol('Style','Radio', 'Units','normalized','Parent',upind, ...
    'HandleVisibility','off', 'Position',[0.05 9/17 0.9 3/17], ...
    'String','DKI tractography (Step 1 - Find KT maximuns and new metrics)',...
    'Tag','tDKI1','FontWeight','bold')
uicontrol('Style','Radio', 'Units','normalized','Parent',upind, ...
    'HandleVisibility','off', 'Position',[0.05 5/17 0.9 3/17], ...
    'String','DKI tractography (Step 2 - Tract)',...
    'Tag','tDKI2','FontWeight','bold')
uicontrol('Style','Radio', 'Units','normalized','Parent',upind, ...
    'HandleVisibility','off', 'Position',[0.05 1/17 0.9 3/17], ...
    'String','DKI tractography (Step 1 and 2)', 'Tag','tDKI12','FontWeight','bold')

    function Call_Fit(source, eventdata)
        % Resume and close
        
        switch get(get(upind,'SelectedObject'),'Tag')
            case 'tDTI'
                % Call DTI tractography
                p={'V1 path:',...
                    'Seeds mask:',...
                    'Seeds mask threshold:',...
                    'Allowed tracking Region:',...
                    'Tracking region threshold:',...
                    'Angular deviation  threshold:',...
                    'Min tract length (cm):',...
                    'Max tract length (cm):',...
                    'N seeds per voxel:'};
                
                if strcmp(st_comp,'win')
                    def={'C:\path\Subject1\DTImetrics\DWI_brain_GF150_nlsDTI_DT_v1.nii',...
                        'C:\path\Subject1\DTImetrics\DWI_brain_GF150_nlsDTI_DT_FA.nii',...
                        '0.05',...
                        'C:\path\Subject1\DTImetrics\DWI_brain_GF150_nlsDTI_DT_FA.nii',...
                        '0.05',...
                        '45',...
                        '50',...
                        '600',...
                        '1'};
                else
                    def={'/path/Subject1/DTImetrics/DWI_brain_EM_GF150_nlsDTI_DT_v1.nii',...
                        '/path/Subject1/DTImetrics/DWI_brain_EM_GF150_nlsDTI_DT_FA.nii',...
                        '0.05',...
                        '/path/Subject1/DTImetrics/DWI_brain_EM_GF150_nlsDTI_DT_FA.nii',...
                        '0.05',...
                        '45',...
                        '50',...
                        '600',...
                        '1'};
                end
                r=inputdlg(p,'',1,def);
                V1_name=r{1};
                S_name=r{2};
                tS=str2double(r{3});
                FA_name=r{4};
                tFA=str2double(r{5});
                ta=str2double(r{6});
                tmin=str2double(r{7});
                tmax=str2double(r{8});
                NoS=str2double(r{9});
                fun_DTI_tractography(V1_name,S_name,tS,FA_name,tFA,...
                    ta,tmin,tmax,NoS)
            case 'tDKI1'
                % Call first step of DKI tractography
                p={'Diffusion tensor:',...
                    'Kurtosis tensor:',...
                    'Brain mask:',...
                    'Method: 1- fibers from KT, 2- fibers from DKI-ODF:'};
                if strcmp(st_comp,'win')
                    def={'C:\path\Subject1\DWI_brain_GF150_olsDKI_DT.nii',...
                        'C:\path\Subject1\DWI_brain_GF150_olsDKI_KT.nii',...
                        'C:\path\Subject1\DWI_brain_mask.nii',...
                        '1'};
                else
                    def={'/path/Subject1/DWI_brain_GF150_olsDKI_DT.nii',...
                        '/path/Subject1/DWI_brain_GF150_olsDKI_KT.nii',...
                        '/path/Subject1/DWI_brain_mask.nii',...
                        '1'};
                end
                r=inputdlg(p,'',1,def);
                DKI_DT=r{1};
                DKI_KT=r{2};
                Mask_n=r{3};
                me=str2double(r{4});
                if me==1
                    p={'MD fraction threshold:'};
                    def={'0.8'};
                    r0=inputdlg(p,'',1,def);
                    fMD=str2double(r0{1});
                    fun_extract_directions_from_DKI_KT(DKI_DT,DKI_KT,Mask_n,fMD)
                elseif me==2
                    p={'Alfa:'};
                    def={'4'};
                    r0=inputdlg(p,'',1,def);
                    alfa=str2double(r0{1});
                    fun_extract_directions_from_DKI_ODF(DKI_DT,DKI_KT,Mask_n,alfa)
                end
                
            case 'tDKI2',
                % Call second step of DKI tractography
                p={'FDir path:',...
                    'NM path:',...
                    'Seeds mask:',...
                    'Seeds mask threshold:',...
                    'Allowed tracking Region:',...
                    'Tracking region threshold:',...
                    'Angular deviation  threshold:',...
                    'Min tract length (cm):',...
                    'Max tract length (cm):',...
                    'N seeds per voxel:'};
                if strcmp(st_comp,'win')
                    def={'C:\path\Subject1\DKI_tract_m\DWI_brain_GF150_olsDKI_KT_FDir.nii',...
                        'C:\path\Subject1\DKI_tract_m\DWI_brain_GF150_olsDKI_KT_NM.nii',...
                        'C:\path\Subject1\DTImetrics\DWI_brain_GF150_olsDKI_DT_FA.nii',...
                        '0.05',...
                        'C:\path\Subject1\DTImetrics\DWI_brain_GF150_olsDKI_DT_FA.nii',...
                        '0.05',...
                        '45',...
                        '50',...
                        '600',...
                        '1'};
                else
                    def={'/path/Subject1/DKI_tract_m/DWI_brain_GF150_olsDKI_KT_FDir.nii',...
                        '/path/Subject1/DKI_tract_m/DWI_brain_GF150_olsDKI_KT_NM.nii',...
                        '/path/Subject1/DTImetrics/DWI_brain_GF150_olsDKI_DT_FA.nii',...
                        '0.05',...
                        '/path/Subject1/DTImetrics/DWI_brain_GF150_olsDKI_DT_FA.nii',...
                        '0.05',...
                        '45',...
                        '50',...
                        '600',...
                        '1'};
                end
                r=inputdlg(p,'',1,def);
                FDir_name=r{1};
                NM_name=r{2};
                S_name=r{3};
                tS=str2double(r{4});
                FA_name=r{5};
                tFA=str2double(r{6});
                ta=str2double(r{7});
                tmin=str2double(r{8});
                tmax=str2double(r{9});
                NoS=str2double(r{10});
                fun_DKI_tractography(FDir_name,NM_name,S_name,tS,FA_name,tFA,...
                    ta,tmin,tmax,NoS)
                
            case 'tDKI12',
                % calls tractpgraphy step 1 and 2 simultaneously 
                p={'DKI Diffusion tensor:',...
                    'DKI Kurtosis tensor:',...
                    'Brain mask:',...
                    'Method: 1- fibers from KT, 2- fibers from DKI-ODF:',...
                    'Seeds mask:',...
                    'Seeds mask threshold:',...
                    'Allowed tracking Region:',...
                    'Tracking region threshold:',...
                    'Angular deviation  threshold:',...
                    'Min tract length (cm):',...
                    'Max tract length (cm):',...
                    'N seeds per voxel:'};
                if strcmp(st_comp,'win')
                    def={'C:\path\Subject1\DWI_brain_GF150_olsDKI_DT.nii',...
                        'C:\path\Subject1\DWI_brain_GF150_olsDKI_KT.nii',...
                        'C:\path\Subject1\DWI_brain_mask.nii',...
                        '1',...
                        'C:\path\Subject1\DTImetrics\DWI_brain_GF150_olsDTI_DT_FA.nii',...
                        '0.05',...
                        'C:\path\Subject1\DTImetrics\DWI_brain_GF150_olsDTI_DT_FA.nii',...
                        '0.05',...
                        '45',...
                        '50',...
                        '600',...
                        '1'};
                else
                    def={'/path/Subject1/DWI_brain_GF150_olsDKI_DT.nii',...
                        '/path/Subject1/DWI_brain_GF150_olsDKI_KT.nii',...
                        '/path/Subject1/DWI_brain_mask.nii',...
                        '1',...
                        '/path/Subject1/DTImetrics/DWI_brain_GF150_olsDKI_DT_FA.nii',...
                        '0.05',...
                        '/path/Subject1/DTImetrics/DWI_brain_GF150_olsDKI_DT_FA.nii',...
                        '0.05',...
                        '45',...
                        '50',...
                        '600',...
                        '1'};
                end
                r=inputdlg(p,'',1,def);
                DKI_DT=r{1};
                DKI_KT=r{2};
                Mask_n=r{3};
                me=str2double(r{4});
                S_name=r{5};
                tS=str2double(r{6});
                FA_name=r{7};
                tFA=str2double(r{8});
                ta=str2double(r{9});
                tmin=str2double(r{10});
                tmax=str2double(r{11});
                NoS=str2double(r{12});
                if me==1
                    p={'MD fraction threshold:'};
                    def={'0.8'};
                    r0=inputdlg(p,'',1,def);
                    fMD=str2double(r0{1});
                    fun_extract_directions_from_DKI_KT(DKI_DT,DKI_KT,Mask_n,fMD)
                    [pth,outname, daa]=fileparts(DKI_KT);
                    FDir_name=[pth,'/DKI_tract_m/', outname, '_DKIKT_MDC',num2str(fMD*100),'_FDir.nii'];;
                    NM_name=[pth,'/DKI_tract_m/', outname, '_DKIKT_MDC',num2str(fMD*100),'_NM.nii'];
               elseif me==2
                    p={'Alfa:'};
                    def={'4'};
                    r0=inputdlg(p,'',1,def);
                    alfa=str2double(r0{1});
                    fun_extract_directions_from_DKI_ODF(DKI_DT,DKI_KT,Mask_n,alfa)
                    [pth,outname, daa]=fileparts(DKI_KT);
                    FDir_name=[pth,'/DKI_tract_m/', outname, '_DKIODF_alfa',num2str(alfa),'_FDir.nii'];
                    NM_name=[pth,'/DKI_tract_m/', outname, '_DKIODF_alfa',num2str(alfa),'_NM.nii'];
                end
                fun_DKI_tractography(FDir_name,NM_name,S_name,tS,FA_name,tFA,...
                    ta,tmin,tmax,NoS)
        end
        uiresume(hfig)
        close(hfig)
    end
uiwait(hfig)
end