function Groupbar(dat,Groups,Labs)
%%% Written by Aaron Schultz (aschultz@martinos.org)
%%%
%%% Copyright (C) 2012,  Aaron P. Schultz
%%%
%%% Supported in part by the NIH funded Harvard Aging Brain Study (P01AG036694) and NIH R01-AG027435 
%%%
%%% This program is free software: you can redistribute it and/or modify
%%% it under the terms of the GNU General Public License as published by
%%% the Free Software Foundation, either version 3 of the License, or
%%% any later version.
%%% 
%%% This program is distributed in the hope that it will be useful,
%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%% GNU General Public License for more details.

tt = unique(Groups);
for ii = tt(:)'
    ind = find(Groups==ii);

    mm(ii) = mean(dat(ind));
    sd(ii) = std(dat(ind));
    se(ii) = (std(dat(ind))./sqrt(numel(ind)))*1.96;
end

cols = jet(numel(mm));
% cols = [0 0 1; 1 0 1; 1 0 0];
% if mod(numel(mm),2)==1
%     cols = colmap('cbd_pu_gn',numel(mm));
%     %cols=cols(setdiff(1:numel(mm),ceil((numel(mm)/2))),:);
% else
%     cols = colmap('cbd_pu_gn',numel(mm)+2);
%     cols=cols(setdiff(1:size(cols,1),(numel(mm)/2)+1:(numel(mm)/2)+2),:);
% end

for ii = 1:length(mm)
    bar(ii,mm(ii),'FaceColor',cols(ii,:)); hold on
end
% bar((1:length(mm)),mm); hold on
% errorbar(1:length(mm),mm,2*sd,'.k','linewidth',1.5)
h=errorbar((1:length(mm)),mm,se,'k.','linewidth',3); hold on;
% errorbar_tick(h,10); hold on

set(gca,'XTick', 1:length(Labs), 'XTickLabel',Labs,'FontSize',14);

ax = axis;
ax(1:2) = [.5 numel(tt)+.5];
axis(ax);
if ax(3)~=0 && ax(4)~=0
    plot(ax(1:2),[0 0],'m--','linewidth',2);
end
end

function errorbar_tick(h,w,xtype)
%ERRORBAR_TICK Adjust the width of errorbars
%   ERRORBAR_TICK(H) adjust the width of error bars with handle H.
%      Error bars width is given as a ratio of X axis length (1/80).
%   ERRORBAR_TICK(H,W) adjust the width of error bars with handle H.
%      The input W is given as a ratio of X axis length (1/W). The result 
%      is independent of the x-axis units. A ratio between 20 and 80 is usually fine.
%   ERRORBAR_TICK(H,W,'UNITS') adjust the width of error bars with handle H.
%      The input W is given in the units of the current x-axis.
%
%   See also ERRORBAR
%

% Author: Arnaud Laurent
% Creation : Jan 29th 2009
% MATLAB version: R2007a
%
% Notes: This function was created from a post on the french forum :
% http://www.developpez.net/forums/f148/environnements-developpement/matlab/
% Author : Jerome Briot (Dut) 
%   http://www.mathworks.com/matlabcentral/newsreader/author/94805
%   http://www.developpez.net/forums/u125006/dut/
% It was further modified by Arnaud Laurent and Jerome Briot.

% Check numbers of arguments
error(nargchk(1,3,nargin))

% Check for the use of V6 flag ( even if it is depreciated ;) )
flagtype = get(h,'type');

% Check number of arguments and provide missing values
if nargin==1
	w = 80;
end

if nargin<3
   xtype = 'ratio';
end

% Calculate width of error bars
if ~strcmpi(xtype,'units')
    dx = diff(get(gca,'XLim'));	% Retrieve x limits from current axis
    w = dx/w;                   % Errorbar width
end

% Plot error bars
if strcmpi(flagtype,'hggroup') % ERRORBAR(...)
    
    hh=get(h,'children');		% Retrieve info from errorbar plot
    x = get(hh(2),'xdata');		% Get xdata from errorbar plot
    
    x(4:9:end) = x(1:9:end)-w/2;	% Change xdata with respect to ratio
    x(7:9:end) = x(1:9:end)-w/2;
    x(5:9:end) = x(1:9:end)+w/2;
    x(8:9:end) = x(1:9:end)+w/2;

    set(hh(2),'xdata',x(:))	% Change error bars on the figure

else  % ERRORBAR('V6',...)
    
    x = get(h(1),'xdata');		% Get xdata from errorbar plot
    
    x(4:9:end) = x(1:9:end)-w/2;	% Change xdata with respect to the chosen ratio
    x(7:9:end) = x(1:9:end)-w/2;
    x(5:9:end) = x(1:9:end)+w/2;
    x(8:9:end) = x(1:9:end)+w/2;

    set(h(1),'xdata',x(:))	% Change error bars on the figure
    
end
end
